#version 330
#ifdef TEXTURE
uniform sampler2D textures[8];
in vec2 texcoord_out;
uniform float alphaRef;
#endif // TEXTURE

// in float fogFactor;
uniform float fogDensity;
uniform bool fog;
in vec3 cameraToFragment;
// in float fragmentDistance;
uniform vec3 fogColor;
uniform bool glow;
uniform float alpha = 1.0;
uniform float lineWidth;

#ifdef LIT
struct lightSourceParameters
{
	vec4 ambient;              // Aclarri
	vec4 diffuse;              // Dcli
	vec4 specular;             // Scli
	vec3 position;             // Ppli
	vec3 halfVector;           // Derived: Hi
};

uniform lightSourceParameters lightSource[4];
in vec3 fragNormal;
#endif // LIT

in vec4 frontColor;
out vec4 fragColor[2];

in vec4 worldPosition;

float grid(float inVal)
{
	float distanceFromGrid = mod( inVal, 128.0 );
	if ( distanceFromGrid > 64.0 )
		distanceFromGrid = 128 - distanceFromGrid;

	// return distanceFromGrid > lineWidth ? 0.0 : 1.0;

	return smoothstep( 1.0 * lineWidth, 0.25f * lineWidth, distanceFromGrid );
}

void main(void)
{
	vec4 color = frontColor;
#ifdef TEXTURE
	vec4 textureSample = texture(textures[0], texcoord_out.st);
	if ( textureSample.a < alphaRef )
		discard;
	color *= textureSample;
#endif // TEXTURE

	float gridAmt = max( grid(worldPosition.x), grid(worldPosition.z) );

	if ( gridAmt < 0.1 )
		discard;

	vec3 finalColor = color.rgb;

	fragColor[0].rgba = vec4(0);
	fragColor[1].rgba = vec4(finalColor, gridAmt * alpha);
}




