#version 330
#ifdef TEXTURE
uniform sampler2D textures[8];
in vec2 texcoord_out;
uniform float alphaRef;
#endif // TEXTURE

// in float fogFactor;
uniform float fogDensity;
uniform bool fog;
in vec3 cameraToFragment;
// in float fragmentDistance;
uniform vec3 fogColor;
uniform bool glow;
uniform float alpha = 1.0;

#ifdef LIT
struct lightSourceParameters
{
	vec4 ambient;              // Aclarri
	vec4 diffuse;              // Dcli
	vec4 specular;             // Scli
	vec3 position;             // Ppli
	vec3 halfVector;           // Derived: Hi
};

uniform lightSourceParameters lightSource[4];
in vec3 fragNormal;
#endif // LIT

in vec4 frontColor;
out vec4 fragColor[2];

in vec4 worldPosition;

void main(void)
{
	vec4 color = frontColor;
#ifdef TEXTURE
	vec4 textureSample = texture(textures[0], texcoord_out.st);
	if ( textureSample.a < alphaRef )
		discard;
	color *= textureSample;
#endif // TEXTURE
	color.a *= alpha;

#ifdef LIT
	// vec3 n = normalize(fragNormal);
	// float NdotL = max(dot(n,lightSource[0].position)+1.0,0.0) * 0.5;

	// if ( NdotL < 0.4 )
	// 	NdotL = 0.2;

	// vec4 ambientPart = color * lightSource[0].ambient;
	// vec4 diffusePart = color * lightSource[0].diffuse * NdotL;
	// color = ambientPart + diffusePart;
#endif // LIT

	vec3 finalColor = color.rgb;
	if ( fog )
	{
		float fragmentDistance = length(cameraToFragment);
		const float LOG2 = 1.442695;
		float fogFactor = exp2( -fogDensity *
				fogDensity *
				fragmentDistance *
				fragmentDistance *
				LOG2 );
		fogFactor = clamp(fogFactor,0.0,1.0);
		finalColor = mix(fogColor.rgb, color.rgb, fogFactor );
	}
	vec3 glowColor = finalColor;

	fragColor[0].rgba = vec4(finalColor, color.a);
	if ( glow )
		fragColor[1].rgba = vec4(finalColor, color.a);
	else
		fragColor[1].rgba = vec4(0,0,0,color.a);
	// fragColor = vec4(1.0,0.0,0.0,1.0);
}



