#version 330
uniform sampler2D textures[8];
in vec2 texcoord_out;
uniform float alphaRef;

uniform samplerBuffer bufferTexture;

uniform float fogDensity;
uniform bool fog;
uniform vec3 cameraDirection;
uniform vec3 fogColor;
uniform bool glow;
uniform uint globalSeconds;
uniform uint globalMicroseconds;
uniform vec4 bandwidthColor;
uniform vec4 selectedColor;
uniform vec4 tint;
uniform bool cable;

in vec3 fragNormal;

in vec4 frontColor;
out vec4 fragColor[2];

in vec4 worldPosition;

void main(void)
{
	vec4 color = frontColor * bandwidthColor;
	float bandwidthValueA = texelFetch(bufferTexture, int(floor(texcoord_out.t)) - 1).r;
	float bandwidthValueB = texelFetch(bufferTexture, int(floor(texcoord_out.t))).r;

	float abmix = fract(texcoord_out.t);
	float bandwidthValue = mix( bandwidthValueA, bandwidthValueB, abmix);

	vec3 zeroBandwidthColor = vec3(0.06,0.025,0.15);
	vec3 partialBandwidthColor = vec3(0.125,0.22,0.25);
	vec3 fullBandwidthColor = vec3(0.5,0.9,1.0);

	zeroBandwidthColor.rgb = mix( zeroBandwidthColor.rgb, tint.rgb, tint.a );
	partialBandwidthColor.rgb = mix( partialBandwidthColor.rgb, tint.rgb, tint.a );
	fullBandwidthColor.rgb = mix( fullBandwidthColor.rgb, tint.rgb, tint.a );

	bandwidthValue = clamp(bandwidthValue, 0.0, 1.0);
	color.rgb = zeroBandwidthColor;

	vec3 normal = normalize(fragNormal);
	vec3 glowColor = color.rgb;

	{
		float threshhold = mix(0.95, 0.5, bandwidthValue);
		if ( normal.y > threshhold )
			glowColor.rgb = color.rgb = fullBandwidthColor;
		else if ( normal.y > 0.5 )
			glowColor.rgb = color.rgb = partialBandwidthColor;
	}
	if ( !cable )
	{
		if ( abmix > 0.8 )
		{
			color.rgb = zeroBandwidthColor;
			glowColor.rgb = vec3(0,0,0);
		}
	}
	color.a = 1.0;
	color.rgb += selectedColor.rgb;

	vec3 finalColor = color.rgb;

	float tSeconds = float(mod(globalSeconds,3u)) + (float(globalMicroseconds) / 1000000.0);
	float f = tSeconds / 3.0;
	f = sin( 2.0 * 3.1415927 * f ) * 0.1 + 0.1;

	fragColor[0].rgba = vec4(finalColor, color.a);
	fragColor[1].rgba = vec4(mix(vec3(0),glowColor,f), color.a);
}


