#version 330
in mat4 localToWorldAttrib;
uniform bool fog;
uniform float fogDensity;
// uniform mat4 localToWorld;
uniform mat4 worldToView;
uniform mat4 viewToProjection;
uniform float worldOffset;
uniform float invWorldDims;
uniform vec3 cameraPosition;

// out float fogFactor;
out vec3 cameraToFragment;
out vec3 reflectDirection;
out vec3 local_light_pos;
out vec3 local_normal;
out vec4 viewPosition;
in vec4 vertex;
/* in vec3 normal; */
uniform vec4 universal_color;

// uniform float globalTime;
uniform uint globalSeconds;
uniform uint globalMicroseconds;
out float timer;
out vec2 fragTexCoord[4];
out vec4 frontColor;
out vec2 texel;
out float lightingAlpha;
out vec4 worldPosition;

struct lightSourceParameters
{
	vec4 ambient;              // Aclarri
	vec4 diffuse;              // Dcli
	vec4 specular;             // Scli
	vec3 position;             // Ppli
	vec3 halfVector;           // Derived: Hi
};

uniform lightSourceParameters lightSource[4];

float timeLoopedAt(uint wrapSeconds)
{
	uint seconds = globalSeconds % wrapSeconds;
	float time = float(seconds) + float(globalMicroseconds)/1000000.0;
	return time;
}

void main(void)
{
	frontColor = universal_color;

	worldPosition = localToWorldAttrib * vertex;
	gl_Position = viewToProjection * worldToView * worldPosition;
	viewPosition = gl_Position;

	float off = 0.5/256.0;
	texel = (worldPosition.xz - vec2(worldOffset)) * invWorldDims + vec2(off);

	vec4 camToFragment = worldToView * worldPosition;
	cameraToFragment = camToFragment.rgb;
	float f = smoothstep( 4000.0, 10.0, cameraPosition.y );
	lightingAlpha = f * 0.2;

	local_light_pos = normalize(worldToView * vec4(lightSource[0].position.xyz, 0.0)).xyz;

	// Okay, let's figure out where our loops are.  In theory, each of these lookups should
	// eventually loop back around.  Let's figure out where.
	//
	// So for each axis, we loop whenever 'globalTime * (...)' == 1.  As our
	// lookups all wrap around at 1.  SO, for our first texture coordinate, we
	// loop at:
	//
	// On X, we loop at Time * 0.005 * 1 == 1.
	// Or:  Time = 1 / 0.005
	// Or:  Time = 200 / 1
	// Or:  Time = 200
	//
	// On Z, we loop at Time * 0.005 * 4 == 1
	// Or:  Time * 4 = 200
	// Or:  Time = 200/4
	// Or:  Time = 50.
	//
	// SO.  If we're gonna be smart, we will wrap both at time 200.

	float aTime = timeLoopedAt( 200u );

	// 200 / 1.5 == 133.333333
	// 1.0/0.7
	// 1.428571
	// Gives us 200/1.428571 == 140
	// 200 / 2.5 == 80.0
	// Minimum common is 1500
	//
	// 140 -> 7,5,2,2
	// 80 -> 5,2,2,2,2
	//
	// Least common multiple == 2*2*2*2*5*7 == 460
	float bTime = timeLoopedAt( 460u );

	// 100 / 1.0 = 100
	// 100 / 4.0 = 25
	float cTime = timeLoopedAt( 100u );

	// 100 / 5.0 = 20
	// 100 / 2.0 = 50
	float dTime = timeLoopedAt( 100u );

	timer = dTime / 100.0;

	fragTexCoord[0] = worldPosition.xz * 0.0002 + vec2(-1.0,-4.0) * aTime * 0.005;
	fragTexCoord[1] = worldPosition.zx * 0.0001 + vec2(-1.428571,-2.5) * bTime * 0.005;
	fragTexCoord[2] = worldPosition.xz * 0.004 + vec2(1.0,4.0) * cTime * 0.01;
	fragTexCoord[3] = worldPosition.zx * 0.003 + vec2(5.0,2.0) * dTime * 0.01;
}

